<?php
// Own namespace
namespace Org\Mxchange\CoreFramework\Stack;

// Import framework stuff
use Org\Mxchange\CoreFramework\Generic\FrameworkInterface;

/**
 * A Stackable interface
 *
 * @author		Roland Haeder <webmaster@shipsimu.org>
 * @version		0.0.0
 * @copyright	Copyright (c) 2007, 2008 Roland Haeder, 2009 - 2022 Core Developer Team
 * @license		GNU GPL 3.0 or any newer version
 * @link		http://www.shipsimu.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
interface Stackable extends FrameworkInterface {
	/**
	 * Pushs a value on a named stacker
	 *
	 * @param	$stackerName	Name of the stacker
	 * @param	$value			Value to push on it
	 * @return	void
	 * @throws	StackerFullException	If the stacker is full
	 */
	function pushNamed (string $stackerName, $value);

	/**
	 * 'Pops' a value from a named stacker and returns it's value
	 *
	 * @param	$stackerName	Name of the stacker
	 * @return	$value			Value of the current stack entry
	 * @throws	BadMethodCallException	If the named stacker was not found
	 * @throws	BadMethodCallException	If the named stacker is empty
	 */
	function popNamed (string $stackerName);

	/**
	 * Get value from named stacker but don't "pop" it
	 *
	 * @param	$stackerName	Name of the stacker
	 * @return	$value			Value of last added value
	 * @throws	BadMethodCallException	If the named stacker was not found
	 * @throws	BadMethodCallException	If the named stacker is empty
	 */
	function getNamed (string $stackerName);

	/**
	 * Checks whether the given stack is initialized (set in array $stackers)
	 *
	 * @param	$stackerName	Name of the stack
	 * @return	$isInitialized	Whether the stack is initialized
	 */
	function isStackInitialized (string $stackerName);

	/**
	 * Checks whether the given stack is empty
	 *
	 * @param	$stackerName	Name of the stack
	 * @return	$isEmpty			Whether the stack is empty
	 * @throws	BadMethodCallException	If given stack is missing
	 */
	function isStackEmpty (string $stackerName);

	/**
	 * Initializes given stacker
	 *
	 * @param	$stackerName	Name of the stack
	 * @param	$forceReInit	Force re-initialization
	 * @return	void
	 * @throws	UnsupportedOperationException	This method is not (and maybe never will be) supported
	 */
	function initStack (string $stackerName, bool $forceReInit = false);

	/**
	 * Initializes all stacks
	 *
	 * @return	void
	 * @throws	UnsupportedOperationException	This method is not (and maybe never will be) supported
	 */
	function initStacks (array $stacks, bool $forceReInit = false);

	/**
	 * Getter for size of given stack (array count)
	 *
	 * @param	$stackerName	Name of the stack
	 * @return	$count			Size of stack (array count)
	 */
	function getStackCount (string $stackerName);

}
