<?php
// Own namespace
namespace Org\Mxchange\CoreFramework\Tests\Resolver\Command;

// Import framework stuff
use Org\Mxchange\CoreFramework\Command\InvalidCommandException;
use Org\Mxchange\CoreFramework\Resolver\Command\BaseCommandResolver;
use Org\Mxchange\CoreFramework\Resolver\Command\CommandResolver;

// Import SPL stuff
use \InvalidArgumentException;

/**
 * A command resolver for console commands
 *
 * @author		Roland Haeder <webmaster@shipsimu.org>
 * @version		0.0.0
 * @copyright	Copyright (c) 2007, 2008 Roland Haeder, 2009 - 2022 Core Developer Team
 * @license		GNU GPL 3.0 or any newer version
 * @link		http://www.shipsimu.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
class TestsConsoleCommandResolver extends BaseCommandResolver implements CommandResolver {
	/**
	 * Protected constructor
	 *
	 * @return	void
	 */
	private function __construct () {
		// Call parent constructor
		parent::__construct(__CLASS__);

		// Set prefix to "TestsConsole"
		$this->setClassPrefix('tests_console');
	}

	/**
	 * Creates an instance of a TestsConsole command resolver with a given default command
	 *
	 * @param	$commandName				The default command we shall execute
	 * @return	$resolverInstance			The prepared command resolver instance
	 * @throws	InvalidArgumentException		Thrown if default command is not set
	 * @throws	InvalidCommandException		Thrown if default command is invalid
	 */
	public static final function createTestsConsoleCommandResolver (string $commandName) {
		// Create the new instance
		$resolverInstance = new TestsConsoleCommandResolver();

		// Is the variable $commandName set and the command is valid?
		if (empty($commandName)) {
			// Then thrown an exception here
			throw new InvalidArgumentException('Parameter "commandName" is empty');
		} elseif ($resolverInstance->isCommandValid('Org\Mxchange\CoreFramework\Tests\Command', $commandName) === false) {
			// Invalid command found
			throw new InvalidCommandException(array($resolverInstance, $commandName), self::EXCEPTION_INVALID_COMMAND);
		}

		// Set namespace for command
		$resolverInstance->setNamespace('Org\Mxchange\CoreFramework\Tests\Command');
		$resolverInstance->setCommandName($commandName);

		// Return the prepared instance
		return $resolverInstance;
	}

}
